G = 9.81 # m/s^2

LPMtCCmPH = 60000.0 # Liter per minute too cubic centimeters per Hour
LPMtCCmPS = 16.666666667 # Liters per minute to cubic centimeters per second
MtCm = 100.0 # Meters to Centimeters
LPMtCMPS = 1.666666666e-5 # Liters per minute to Cubic meters per second
CmtM = 1.0e-2 # centimeter to meter
from math import *
from matplotlib.patches import Circle, Rectangle, Polygon
from matplotlib.transforms import Value
from matplotlib.backends.backend_agg import RendererAgg


# all conversions to meters and seconds
in_conv = {"liters_per_minute": 1.66666666666e-5, # cubic meters per second
           "cubic_meters_per_second": 1.0,
           "centimeters": 1.0e-2, # meters
           "square_meters": 1.0, # leave alone
           "radians": 1.0,
}

# all conversions from meters and seconds
out_conv = {"meters_per_second": 1.0,
            "centimeters_per_second": 1.0e2, # from meters per second
            "centimeters_per_hour": 1.0e2 * 60.0 * 60.0 # from meters per second
}

def units(f):
    def wrapper(*args, **kwds):
        if kwds.has_key("in_units"):
            result = f(*[in_conv[c] * a for c, a in zip(kwds["in_units"],args)])
        else:
            result = f(*args)
        if kwds.has_key("out_units"):
            result = out_conv[kwds["out_units"]] * result
        return result
    return wrapper

@units
def velocity(flowrate, diameter):
    """Velocity as a function of flowrate and tube diameter."""
    # flowrate is in liters per minute (10^3 cm^3 / minute)
    # diameter in centimeters
    # returns meters per second
    return (4 * flowrate)/(pi * (diameter)**2)

@units
def radius(velocity, theta):
    """Radius as a function of velocity and angle."""
    # velocity in meters per second
    # theta (angle from horizontal, radians)
    # returns meters
    v_u = velocity * cos(theta)
    tm = 2 * v_u / G
    return velocity * sin(theta) * tm

@units
def area(radius):
    return pi * radius**2

@units
def depth(sprinkler_flow, area):
    return sprinkler_flow/area

@units
def find_m(n, radius):
    ds = 20.0/n
    m = 2 * floor(radius/ds) + 1
    if m >= n:
        return n
    else:
        return m

def tupleize(alist):
    return list([tuple(item) for item in alist])

def print_params(n, theta):
    f_in = 150.0 # L/min
    d_in = 0.6 # cm 
    print " # _flow_ n_vel radius ___area___ _depth_ _day_ _4days_  m _m*d_"
    for ii in range(1, n+1):
        F = f_in/ii
        V = velocity(F, d_in, in_units=["liters_per_minute", "centimeters"])
        R = radius(V, theta)
        A = area(R)
        D = depth(F, A, in_units=["liters_per_minute", "square_meters"], out_units="centimeters_per_hour")
        M = find_m(ii, R)
        print "%2i %6.2f %5.2f %6.2f %10.2f %5.5f %5.2f %7.3f %2i %4.3f" % (ii, F, V, R, A, D, 24*D, 4*24*D, M, M*D)    
        
def draw_sprinklers(n, theta):
    # draw n sprinklers on a pipe
    width = 80 * 9
    height = 30 * 10
    renderer = RendererAgg(width, height, Value(92))
    bg = Rectangle((0,0), width, height)
    bg.set_zorder(0)
    bg.set_facecolor("#FFFFFF")
    bg.draw(renderer)
    
    # draw farm
    farm_w = 8 * 80 # 8 pixels = 1 meter
    farm_h = 8 * 30
    farm = Rectangle((30,30), farm_w, farm_h)
    farm.set_antialiased(False)
    farm.set_zorder(1)
    farm.set_facecolor("#806C59")
    farm.set_edgecolor("#000000")
    farm.draw(renderer)
    
    # draw pipe
    pipe_w = 1
    pipe_h = 8 * 20
    pipe = Rectangle((width/2, 70), pipe_w, pipe_h)
    pipe.set_zorder(2)
    pipe.set_facecolor("#555555")
    pipe.draw(renderer)

    alpha = 12*depth_to_ground(theta, n)
    r = 8 * radius(velocity(150./n, 0.6), theta)    
    step = 8 * 20. / ( n - 1 )
    for ii in range(0, n):
        y = 70 + ii * step
        x = width/2
        sprinkler = Circle((x,y), r)
        sprinkler.set_zorder(3)
        sprinkler.set_facecolor("#5555AA")
        sprinkler.set_lw(0)
        sprinkler.set_alpha(alpha)
        sprinkler.draw(renderer)
    renderer._renderer.write_png("./farm.png")



